@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2008 The PHP Group                                     |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -29,7 +29,7 @@
 
  */
 
-/* $Id: apc.h,v 3.14 2007/03/17 14:01:41 gopalv Exp $ */
+/* $Id: apc.h,v 3.14.2.2 2008/05/11 18:57:00 rasmus Exp $ */
 
 #ifndef APC_H
 #define APC_H
@@ -59,6 +59,7 @@
 #endif
 
 #include "php.h"
+#include "main/php_streams.h"
 
 /* log levels constants (see apc_log) */
 enum { APC_DBG, APC_NOTICE, APC_WARNING, APC_ERROR };
@@ -92,15 +93,9 @@ extern char** apc_tokenize(const char* s, char delim);
 typedef struct apc_fileinfo_t 
 {
     char fullpath[MAXPATHLEN+1];
-    struct stat st_buf;
+    php_stream_statbuf st_buf;
 } apc_fileinfo_t;
 
-#ifndef PHP_WIN32
-#define apc_stat(f, b) stat(f, b)
-#else
-#define apc_stat(f, b) apc_win32_stat(f, b TSRMLS_CC)
-extern int apc_win32_stat(const char *path, struct stat *buf TSRMLS_DC);
-#endif
 extern int apc_search_paths(const char* filename, const char* path, apc_fileinfo_t* fileinfo);
 
 /* regular expression wrapper functions */
