@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006 The PHP Group                                     |
+  | Copyright (c) 2008 The PHP Group                                     |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,19 +26,24 @@
 
  */
 
-/* $Id: apc_sma.c,v 1.67 2007/08/03 20:53:57 gopalv Exp $ */
+/* $Id: apc_sma.c,v 1.69.2.4 2008/05/11 18:57:00 rasmus Exp $ */
 
 #include "apc_sma.h"
 #include "apc.h"
 #include "apc_globals.h"
 #include "apc_lock.h"
 #include "apc_shm.h"
+#include "apc_cache.h"
 #include <limits.h>
 #if APC_MMAP
 void *apc_mmap(char *file_mask, size_t size);
 void apc_unmap(void* shmaddr, size_t size);
 #endif
 
+#ifdef HAVE_VALGRIND_MEMCHECK_H
+#include <valgrind/memcheck.h>
+#endif
+
 /* {{{ locking macros */
 #define LOCK(c)         { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_lock(c); }
 #define RDLOCK(c)       { HANDLE_BLOCK_INTERRUPTIONS(); apc_lck_rdlock(c); }
@@ -50,7 +55,7 @@ enum { DEFAULT_NUMSEG=1, DEFAULT_SEGSIZE=30*1024*1024 };
 static int sma_initialized = 0;     /* true if the sma has been initialized */
 static unsigned int sma_numseg;     /* number of shm segments to allow */
 static size_t sma_segsize;          /* size of each shm segment */
-static int* sma_segments;           /* array of shm segment ids */
+static size_t* sma_segments;        /* array of shm segment ids */
 static void** sma_shmaddrs;         /* array of shm segment addresses */
 static int sma_lastseg = 0;         /* index of MRU segment */
 
@@ -110,19 +115,12 @@ struct block_t {
 #endif
 
 
-#ifdef max
-#undef max
-#endif
-#define max(a, b) ((a) > (b) ? (a) : (b))
-
-/* {{{ ALIGNWORD: pad up x, aligned to the system's word boundary */
-typedef union { void* p; int i; long l; double d; void (*f)(); } apc_word_t;
-#define ALIGNWORD(x) (sizeof(apc_word_t) * (1 + (((x)-1)/sizeof(apc_word_t))))
+/* {{{ MINBLOCKSIZE */
 #define MINBLOCKSIZE (ALIGNWORD(1) + ALIGNWORD(sizeof(block_t)))
 /* }}} */
 
 /* {{{ sma_allocate: tries to allocate size bytes in a segment */
-static int sma_allocate(void* shmaddr, size_t size)
+static size_t sma_allocate(void* shmaddr, size_t size)
 {
     header_t* header;       /* header of shared memory segment */
     block_t* prv;           /* block prior to working block */
@@ -150,6 +148,11 @@ static int sma_allocate(void* shmaddr, size_t size)
     /* If we have a next fit offset, start searching from there */
     if(header->nfoffset) {
         prv = BLOCKAT(header->nfoffset);
+        /* if prv is the last block, jump to the beginning */
+        if(prv->next == 0) {
+            prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
+            wrapped = 1;
+        }
     } else {    
         prv = BLOCKAT(ALIGNWORD(sizeof(header_t)));
     }
@@ -234,7 +237,7 @@ static int sma_allocate(void* shmaddr, size_t size)
 /* }}} */
 
 /* {{{ sma_deallocate: deallocates the block at the given offset */
-static int sma_deallocate(void* shmaddr, int offset)
+static size_t sma_deallocate(void* shmaddr, size_t offset)
 {
     header_t* header;   /* header of shared memory segment */
     block_t* cur;       /* the new block to insert */
@@ -333,7 +336,7 @@ void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
 
     sma_segsize = segsize > 0 ? segsize : DEFAULT_SEGSIZE;
 
-    sma_segments = (int*) apc_emalloc(sma_numseg*sizeof(int));
+    sma_segments = (size_t*) apc_emalloc(sma_numseg*sizeof(size_t));
     sma_shmaddrs = (void**) apc_emalloc(sma_numseg*sizeof(void*));
     
     for (i = 0; i < sma_numseg; i++) {
@@ -404,7 +407,7 @@ void apc_sma_cleanup()
 /* {{{ apc_sma_malloc */
 void* apc_sma_malloc(size_t n)
 {
-    int off;
+    size_t off;
     int i;
 
     TSRMLS_FETCH();
@@ -416,6 +419,9 @@ void* apc_sma_malloc(size_t n)
         void* p = (void *)(((char *)(sma_shmaddrs[sma_lastseg])) + off);
         if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) += n; }
         UNLOCK(((header_t*)sma_shmaddrs[sma_lastseg])->sma_lock);
+#ifdef VALGRIND_MALLOCLIKE_BLOCK
+        VALGRIND_MALLOCLIKE_BLOCK(p, n, 0, 0);
+#endif
         return p;
     }
     UNLOCK(((header_t*)sma_shmaddrs[sma_lastseg])->sma_lock);
@@ -431,6 +437,9 @@ void* apc_sma_malloc(size_t n)
             if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) += n; }
             UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
             sma_lastseg = i;
+#ifdef VALGRIND_MALLOCLIKE_BLOCK
+            VALGRIND_MALLOCLIKE_BLOCK(p, n, 0, 0);
+#endif
             return p;
         }
         UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
@@ -485,6 +494,9 @@ void apc_sma_free(void* p)
             d_size = sma_deallocate(sma_shmaddrs[i], offset);
             if (APCG(mem_size_ptr) != NULL) { *(APCG(mem_size_ptr)) -= d_size; }
             UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
+#ifdef VALGRIND_FREELIKE_BLOCK
+            VALGRIND_FREELIKE_BLOCK(p, 0);
+#endif
             return;
         }
         UNLOCK(((header_t*)sma_shmaddrs[i])->sma_lock);
